﻿Imports Windows.Storage
Imports Windows.UI.StartScreen

Public NotInheritable Class MainPage
  Inherits Page

  Private Const DataFolderString = """DataFolder"""
  Private Const ErrorTitle = "エラー"
  Private LocalSettings As ApplicationDataContainer =
    ApplicationData.Current.LocalSettings
  Private LocalFolder As StorageFolder =
    ApplicationData.Current.LocalFolder
  Private AllFile As New List(Of String)
  Private Candidate As New List(Of String)
  Private History As New List(Of String)
  Private Rnd As New System.Random()
  Private Sw As New Stopwatch()

  Protected Overrides Async Sub OnNavigatedTo(
    e As Navigation.NavigationEventArgs)

    ScrollViewerMain.HorizontalScrollBarVisibility =
      ScrollBarVisibility.Auto
    ScrollViewerMain.VerticalScrollBarVisibility =
      ScrollBarVisibility.Auto
    SetZoomButton()

    Dim parameter = CType(e.Parameter, String)
    If parameter <> "" Then
      DataFolderPath = parameter
    Else
      If Not LocalSettings.Values(
        DataFolderString) Is Nothing Then
        DataFolderPath =
          LocalSettings.Values(DataFolderString)
      End If
    End If
  End Sub

  Private Sub SetZoomButton()
    ButtonOriginalSize.IsEnabled = Not (
      ScrollViewerMain.ZoomFactor > 0.999F AndAlso
      ScrollViewerMain.ZoomFactor < 1.001F)
  End Sub

  Private Async Sub ButtonSetFolder_Click(
    sender As Object, e As RoutedEventArgs
    ) Handles ButtonSetFolder.Click

    Dim s = TextBoxFolder.Text
    If s = "" Then
      Await ShowMessage(
        "フォルダー名が入力されていません",
        ErrorTitle)
      Exit Sub
    End If

    Dim sFolder As StorageFolder
    Try
      sFolder = Await StorageFolder.
        GetFolderFromPathAsync(s)
    Catch ex As Exception
      ShowMessage("ButtonSetFolder_Click : " &
        ex.Message, ErrorTitle)
      Exit Sub
    End Try

    DataFolderPath = sFolder.Path
  End Sub

  Private Async Function ShowMessage(
    message As String) As Task

    Dim md = New Windows.UI.Popups.MessageDialog(
      message)
    Await md.ShowAsync()
  End Function

  Private Async Function ShowMessage(
    message As String, title As String) As Task

    Dim md = New Windows.UI.Popups.MessageDialog(
      message)
    md.Title = title
    Await md.ShowAsync()
  End Function

  Private DataFolderPathValue As String = ""
  Private Property DataFolderPath As String
    Get
      Return DataFolderPathValue
    End Get
    Set(value As String)
      LoadList(value, False)
      DataFolderPathValue = value
      LocalSettings.Values(DataFolderString) = value
      TextBoxFolder.Text = value
      ButtonAddTile.IsEnabled = value <> ""
    End Set
  End Property

  Private Async Sub LoadList(path As String,
    ByVal mustUpdate As Boolean)

    Dim listString = ""
    If Not mustUpdate Then
      Dim listFileName = ConvertPath(path, True)
      Dim sFile As StorageFile
      Try
        sFile = Await LocalFolder.GetFileAsync(
          listFileName)
        listString = Await FileIO.ReadTextAsync(sFile)
      Catch ex As Exception
        mustUpdate = True
      End Try
    End If

    AllFile.Clear()
    If mustUpdate Then
      Await GetFileList(path)
    Else
      AllFile.AddRange(listString.Split({vbCrLf},
        System.StringSplitOptions.RemoveEmptyEntries))
    End If

    ClearHistory()
    ImageMain.Source = Nothing

    If AllFile.Count > 0 Then
      TextBlockInformation.Text = AllFile.Count &
        " files in " & path
      ButtonRandom.IsEnabled = True
    Else
      ButtonRandom.IsEnabled = False
    End If
  End Sub

  Private Function ConvertPath(path As String,
    addExtension As Boolean) As String

    path = path.Replace(":\", "__")
    path = path.Replace("\", "_")
    If addExtension Then path &= ".txt"
    Return path
  End Function

  Private Async Function GetFileList(
    path As String) As Task

    Await ShowMessage(path & "の探索を始めます")
    Sw.Reset()
    Sw.Start()

    Dim sFolder As StorageFolder
    Try
      sFolder = Await StorageFolder.
        GetFolderFromPathAsync(path)
    Catch ex As Exception
      ShowMessage("GetFileList : " & ex.Message,
        ErrorTitle)
      Exit Function
    End Try

    Dim items = Await sFolder.GetItemsAsync()
    Dim extensions() = New String() {
      ".jpg", ".jpeg", ".bmp"}

    For Each item In items
      For Each extension In extensions
        If System.IO.Path.GetExtension(item.Name).
          ToLower() = extension Then
          AllFile.Add(item.Name)
          Exit For
        End If
      Next
    Next
    items = Nothing

    Dim sFile As StorageFile =
      Await LocalFolder.CreateFileAsync(
      ConvertPath(path, True),
      CreationCollisionOption.ReplaceExisting)
    Await FileIO.WriteLinesAsync(sFile, AllFile)

    Sw.Stop()
    Await ShowMessage(path & "の探索終了（" &
      Sw.ElapsedMilliseconds & "ミリ秒）")
  End Function

  Private Sub ClearHistory()
    History.Clear()
    Candidate.Clear()
    Candidate.AddRange(AllFile)
    HistoryIndex = 0
  End Sub

  Private HistoryIndexValue As Integer
  Private Property HistoryIndex
    Get
      Return HistoryIndexValue
    End Get
    Set(value)
      HistoryIndexValue = value
      If History.Count < 1 Then
        ButtonPrevious.IsEnabled = False
        ButtonNext.IsEnabled = False
      Else
        ButtonPrevious.IsEnabled =
          History.Count - 1 + HistoryIndex > 0
        ButtonNext.IsEnabled = HistoryIndex < 0
      End If
    End Set
  End Property

  Private Async Sub ButtonRandom_Click(sender As Object,
    e As RoutedEventArgs) Handles ButtonRandom.Click

    Dim index = Rnd.Next(Candidate.Count)
    Dim fileName = Candidate(index)
    LoadImage(System.IO.Path.Combine({DataFolderPath,
      fileName}))
    History.Add(fileName)
    Candidate.RemoveAt(index)
    HistoryIndex = 0

    If Candidate.Count < 1 Then
      Await ShowMessage("すべての画像を表示しました" &
        "。初期状態に戻します")
      ClearHistory()
    End If
  End Sub

  Private Async Sub LoadImage(fileName As String)
    Dim sFile As StorageFile
    Try
      sFile = Await StorageFile.
        GetFileFromPathAsync(fileName)
    Catch ex As Exception
      ShowMessage("LoadImage : " & ex.Message,
        ErrorTitle)
      Exit Sub
    End Try

    Dim stream = Await sFile.OpenAsync(
      FileAccessMode.Read)
    Dim bi As New BitmapImage()
    bi.SetSource(stream)
    ImageMain.Source = bi
    TextBlockInformation.Text = fileName

    With ScrollViewerMain
      .ScrollToHorizontalOffset(0.0)
      .ScrollToVerticalOffset(0.0)
      .ZoomToFactor(1.0F)
    End With
  End Sub

  Private Sub ButtonUpdate_Click(sender As Object,
    e As RoutedEventArgs) Handles ButtonUpdate.Click

    LoadList(DataFolderPath, True)
  End Sub

  Private Sub ButtonPrevious_Click(sender As Object,
    e As RoutedEventArgs) Handles ButtonPrevious.Click

    HistoryIndex -= 1
    Dim fileName = History(History.Count - 1 +
      HistoryIndex)
    LoadImage(Path.Combine({DataFolderPath, fileName}))
  End Sub

  Private Sub ButtonNext_Click(sender As Object,
    e As RoutedEventArgs) Handles ButtonNext.Click

    HistoryIndex += 1
    Dim fileName = History(History.Count - 1 +
      HistoryIndex)
    LoadImage(Path.Combine({DataFolderPath, fileName}))
  End Sub

  Private Sub ButtonOriginalSize_Click(
    sender As Object, e As RoutedEventArgs
    ) Handles ButtonOriginalSize.Click

    ScrollViewerMain.ZoomToFactor(1.0F)
  End Sub

  Private Sub ScrollViewerMain_ViewChanged(
    sender As Object,
    e As ScrollViewerViewChangedEventArgs
    ) Handles ScrollViewerMain.ViewChanged

    SetZoomButton()
  End Sub

  Private Sub ScrollViewerMain_LayoutUpdated(
    sender As Object, e As Object
    ) Handles ScrollViewerMain.LayoutUpdated

    ScrollViewerMain.MaxWidth = Me.ActualWidth
  End Sub

  Private Async Sub ButtonAddTile_Click(
    sender As Object, e As RoutedEventArgs
    ) Handles ButtonAddTile.Click

    If DataFolderPath = "" Then Exit Sub
    Dim s = ConvertPath(DataFolderPath, False)
    Dim st As New SecondaryTile()

    st.TileId = s
    st.ShortName = s
    st.DisplayName = s
    st.Arguments = DataFolderPath
    st.TileOptions = TileOptions.ShowNameOnLogo
    st.Logo = New Uri("ms-appx:///Assets/Logo.png")

    Dim isPinned = Await st.RequestCreateAsync()
    If isPinned Then _
      Await ShowMessage("タイルを追加しました")
  End Sub

  Private Sub ButtonExit_Click(sender As Object,
    e As RoutedEventArgs) Handles ButtonExit.Click

    App.Current.Exit()
  End Sub
End Class
